@@ -1,23 +1,23 @@
 import logging
 import math
-import glob
+import yaml
 import random
 import warnings
 from tqdm import tqdm
 
 from dataclasses import dataclass
 from pathlib import Path
-from shutil import copyfile
 from typing import List, Tuple, Optional, Union, Generator, Callable
 
 import cv2
-import shapely
-import matplotlib.patches as patches
-import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import rasterio
 
+import shapely
+import matplotlib.pyplot as plt
+
+import matplotlib.patches as patches
 from matplotlib.patches import Polygon as MplPolygon
 from rasterio.windows import Window
 from shapely.geometry import Polygon, MultiPolygon
@@ -226,10 +226,6 @@ class YoloTiler:
         """Configure logging for the tiler"""
         logger = logging.getLogger('YoloTiler')
         logger.setLevel(logging.INFO)
-        handler = logging.StreamHandler()
-        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
-        handler.setFormatter(formatter)
-        logger.addHandler(handler)
         return logger
 
     def _create_target_folder(self, target: Path) -> None:
@@ -673,8 +669,11 @@ class YoloTiler:
 
                 # Save tile image and labels if include_negative_samples is True or there are labels
                 if self.config.include_negative_samples or tile_labels:
-                    tile_suffix = f'_tile_{tile_idx}{self.config.output_ext}'
-                    self._save_tile(tile_data, image_path, tile_suffix, tile_labels, folder)
+                    # Calculate width and height for the new naming convention
+                    tile_width = abs_x2 - abs_x1
+                    tile_height = abs_y2 - abs_y1
+                    tile_coords = (abs_x1, abs_y1, tile_width, tile_height)
+                    self._save_tile(tile_data, image_path, tile_coords, tile_labels, folder)
 
     def _save_tile_image(self, tile_data: np.ndarray, image_path: Path, suffix: str, folder: str) -> None:
         """
@@ -735,7 +734,7 @@ class YoloTiler:
     def _save_tile(self,
                    tile_data: np.ndarray,
                    original_path: Path,
-                   suffix: str,
+                   tile_coords: Tuple[int, int, int, int],
                    labels: Optional[List],
                    folder: str) -> None:
         """
@@ -744,10 +743,14 @@ class YoloTiler:
         Args:
             tile_data: Numpy array of tile image
             original_path: Path to original image
-            suffix: Suffix for the tile filename
+            tile_coords: Tuple of (x1, y1, width, height) for the tile
             labels: List of labels for the tile
             folder: Subfolder name (train, valid, test)
         """
+        # Create suffix with coordinates: _x_y_width_height
+        x1, y1, width, height = tile_coords
+        suffix = f'_{x1}_{y1}_{width}_{height}{self.config.output_ext}'
+        
         self._save_tile_image(tile_data, original_path, suffix, folder)
         
         # Only save label files for object detection and instance segmentation
@@ -962,7 +965,6 @@ class YoloTiler:
         if self.annotation_type != "image_classification":
             data_yaml = self.source / 'data.yaml'
             if data_yaml.exists():
-                import yaml
                 
                 # Read YAML as structured data
                 with open(data_yaml, 'r') as f:
@@ -986,66 +988,117 @@ class YoloTiler:
 
     def visualize_random_samples(self) -> None:
         """
-        Visualize random samples from the train folder with their annotations.
+        Visualize random samples from the original source images and their corresponding tiles.
+        This helps see how original images are divided into tiles.
         """
         if self.num_viz_samples <= 0:
             return
 
+        # Get images from source directory first
         if self.annotation_type == "image_classification":
             # Get all class directories in train folder
-            train_dir = self.target / 'train'
-            image_paths = list(train_dir.glob(f'**/*{self.config.output_ext}'))
-                
+            train_dir = self.source / 'train'
+            source_image_paths = list(train_dir.glob(f'**/*{self.config.input_ext}'))
         else:
             # Original code for object detection and instance segmentation
-            train_image_dir = self.target / 'train' / 'images'
-            train_label_dir = self.target / 'train' / 'labels'
-            image_paths = list(train_image_dir.glob(f'*{self.config.output_ext}'))
+            train_image_dir = self.source / 'train' / 'images'
+            source_image_paths = list(train_image_dir.glob(f'*{self.config.input_ext}'))
 
-        if not image_paths:
-            self.logger.warning("No images found in train folder for visualization")
+        if not source_image_paths:
+            self.logger.warning("No images found in source train folder for visualization")
             return
-    
-        # Select random samples
-        num_samples = min(self.num_viz_samples, len(image_paths))
-        selected_images = random.sample(image_paths, num_samples)
-
-        # Process each selected image
-        for image_idx, image_path in enumerate(selected_images):
+        
+        # Select random samples from source
+        num_samples = min(self.num_viz_samples, len(source_image_paths))
+        selected_source_images = random.sample(source_image_paths, num_samples)
+        
+        # Process each selected source image
+        for image_idx, source_image_path in enumerate(selected_source_images):
+            # Find all tiles derived from this source image
+            base_name = source_image_path.stem
             
             if self.annotation_type == "image_classification":
-                label_path = image_path.parent.name  # Class name
+                # For image classification
+                class_name = source_image_path.parent.name
+                target_train_dir = self.target / 'train' / class_name
+                tiles = list(target_train_dir.glob(f"{base_name}_*_*_*_*{self.config.output_ext}"))
             else:
                 # For object detection and instance segmentation
-                label_path = train_label_dir / f"{image_path.stem}.txt"
-
-                if not label_path.exists():
-                    self.logger.warning(f"Label file not found for {image_path.name}")
+                target_train_dir = self.target / 'train' / 'images'
+                tiles = list(target_train_dir.glob(f"{base_name}_*_*_*_*{self.config.output_ext}"))
+                
+            if not tiles:
+                self.logger.warning(f"No tiles found for source image {source_image_path.name}")
+                continue
+                
+            # Render source image first
+            if self.annotation_type == "image_classification":
+                source_label = source_image_path.parent.name  # Class name
+            else:
+                # For object detection and instance segmentation
+                source_label_path = source_image_path.parent.parent / 'labels' / f"{source_image_path.stem}.txt"
+                if not source_label_path.exists():
+                    self.logger.warning(f"Label file not found for source {source_image_path.name}")
                     continue
+                
+            # Either the class category or the label file path 
+            label_path = source_label if self.annotation_type == "image_classification" else source_label_path
+            # Render the source image
+            self._render_single_sample(source_image_path, 
+                                       label_path,
+                                       f"{image_idx+1:03d}_source")
+            
+            # Render each tile
+            for tile_idx, tile_path in enumerate(tiles):
+                if self.annotation_type == "image_classification":
+                    tile_label = tile_path.parent.name  # Class name
+                else:
+                    # For object detection and instance segmentation
+                    tile_label_path = self.target / 'train' / 'labels' / f"{tile_path.stem}.txt"
+                    if not tile_label_path.exists():
+                        self.logger.warning(f"Label file not found for tile {tile_path.name}")
+                        continue
+                
+                # Update progress
+                if self.progress_callback:
+                    progress = TileProgress(
+                        current_tile_idx=tile_idx + 1,
+                        total_tiles=len(tiles),
+                        current_set_name=f'rendered (image {image_idx+1}/{num_samples})',
+                        current_image_name=source_image_path.name,
+                        current_image_idx=image_idx + 1,
+                        total_images=num_samples
+                    )
+                    self.progress_callback(progress)
 
-            if self.progress_callback:
-                progress = TileProgress(
-                    current_tile_idx=0,
-                    total_tiles=0,
-                    current_set_name='rendered',
-                    current_image_name=image_path.name,
-                    current_image_idx=image_idx + 1,
-                    total_images=len(selected_images)
-                )
-                self.progress_callback(progress)
-
-            self._render_single_sample(image_path, label_path, image_idx + 1)
-
+                # Extract coordinates from filename for better visualization naming
+                try:
+                    # Parse coordinates from the filename (format: name_x_y_width_height.ext)
+                    parts = tile_path.stem.split('_')
+                    x = parts[-4]
+                    y = parts[-3]
+                    render_id = f"{image_idx+1:03d}_tile_x{x}_y{y}_{tile_idx+1:03d}"
+                except (IndexError, ValueError):
+                    # Fallback if parsing fails
+                    render_id = f"{image_idx+1:03d}_tile_{tile_idx+1:03d}"
+                
+                # Either the class category or the label file path 
+                label_path = tile_label if self.annotation_type == "image_classification" else tile_label_path
+                # Render the tile
+                self._render_single_sample(tile_path, 
+                                           label_path, 
+                                           render_id)
+            
     def _render_single_sample(self, image_path: Path, 
                               label_path: Union[Path, str],  # Path to labels.txt, or class name
-                              idx: int) -> None:
+                              idx: Union[str, int]) -> None:
         """
         Render a single sample with its annotations.
 
         Args:
             image_path: Path to the image file
             label_path: Path to the label file, or class name for image classification
-            idx: Index for the output filename
+            idx: Index or identifier for the output filename
         """
         # Read image using OpenCV
         img = cv2.imread(str(image_path))
@@ -1062,6 +1115,11 @@ class YoloTiler:
         fig, ax = plt.subplots(1)
         ax.imshow(img)
 
+        # Add image filename as figure title
+        is_source = "source" in str(idx)
+        title = f"{'Source: ' if is_source else 'Tile: '}{image_path.name}"
+        fig.suptitle(title, fontsize=10)
+
         # Random colors for different classes
         np.random.seed(42)  # For consistent colors
         colors = np.random.rand(100, 3)  # Support up to 100 classes
@@ -1117,7 +1175,7 @@ class YoloTiler:
                         
         else:  # Image classification
             class_name = label_path
-            ax.text(0.5, 0.5, 
+            ax.text(width / 2, height / 2, 
                     class_name, 
                     fontsize=12, 
                     color='white', 
@@ -1132,8 +1190,8 @@ class YoloTiler:
         plt.tight_layout()
 
         # Save the visualization
-        output_path = self.render_dir / f"sample_{idx:03d}.jpg"
-        plt.savefig(output_path, bbox_inches='tight', pad_inches=0, dpi=300)
+        output_path = self.render_dir / image_path.name
+        plt.savefig(output_path, bbox_inches='tight', pad_inches=0.1, dpi=300)
         plt.close(fig)  # Close the specific figure
 
     def run(self) -> None:
